﻿using Infragistics.Controls;
using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.ComponentModel;
//using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DrawingApp
{ 
    public partial class DiagramRuler : UserControl 
    {
        public DiagramRuler()
        {
            InitializeComponent();
             
            this.Loaded += DiagramRuler_Loaded;
        }

        private void DiagramRuler_Loaded(object sender, RoutedEventArgs e)
        {
            CreateRulers(); 
        }

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            base.OnPropertyChanged(e);

            if (e.Property.Name == "ActualWidth" ||
                e.Property.Name == "ActualHeight" ||
                e.Property.Name == "ActualHeight" ||
                e.Property.Name == "ActualHeight" ||
                e.Property.Name == "ActualHeight" ||
                e.Property.Name == "ActualHeight")
            {
                CreateRulers();
            }
        }

        public static readonly DependencyProperty RulerSizeCountProperty =
         DependencyProperty.Register("RulerSize", typeof(double), typeof(DiagramRuler), new PropertyMetadata(25.0));

        public double RulerSize
        {
            get { return (double)GetValue(RulerSizeCountProperty); }
            set { SetValue(RulerSizeCountProperty, value); }
        }

        public static readonly DependencyProperty MajorThickCountProperty =
           DependencyProperty.Register("MajorThickCount", typeof(int), typeof(DiagramRuler), new PropertyMetadata(5));

        public int MajorThickCount
        {
            get { return (int)GetValue(MajorThickCountProperty); }
            set { SetValue(MajorThickCountProperty, value); }
        }

        public static readonly DependencyProperty MinorThickCountProperty =
           DependencyProperty.Register("MinorThickCount", typeof(int), typeof(DiagramRuler), new PropertyMetadata(50));

        public int MinorThickCount
        {
            get { return (int)GetValue(MinorThickCountProperty); }
            set { SetValue(MinorThickCountProperty, value); }
        }
        //TODO add more Dependency Properties to customize appearance of gridlines and thickmarks 

        public void CreateRulers()
        {
            CreateHorizontalRulers();
            CreateVerticalRulers();
            UpdateRulers();
        }

        public void UpdateRulers()
        {
            if (Diagram == null) return;

            UpdateHorizontalRulers();
            UpdateVerticalRulers();
        }

        private Rectangle CreateVerticalThick(double lenght)
        {
            //TODO use Line instead of Rectangle for better performanc
            var thick = new Rectangle();
            thick.Width = lenght;
            thick.Height = 1;
            thick.Fill = new SolidColorBrush(Colors.Black); 
            return thick;
        }
        private Rectangle CreateHorizontalThick(double lenght)
        {  
            //TODO use Line instead of Rectangle for better performanc
            var thick = new Rectangle();
            thick.Width = 1;
            thick.Height = lenght;
            thick.Fill = new SolidColorBrush(Colors.Black); 
            return thick;
        }
        private Line CreateVerticalLine(double x, double lenght)
        {
            var line = new Line();
            line.StrokeThickness = 1; 
            line.Stroke = new SolidColorBrush(Colors.Black);
            line.X1 = x;
            line.X2 = x;
            line.Y1 = 0;
            line.Y2 = lenght;
            return line;
        }
        private Line CreateHorizontalLine(double y, double lenght)
        {
            var line = new Line();
            line.StrokeThickness = 1;
            line.Stroke = new SolidColorBrush(Colors.Black);
            line.X1 = 0;
            line.X2 = lenght;
            line.Y1 = y;
            line.Y2 = y;
            return line;
        }
          
        internal double MajorIntervalX = 0;
        internal double MinorIntervalX = 0;
        internal double MajorIntervalY = 0;
        internal double MinorIntervalY = 0;

        internal List<TextBlock> LeftLabels = new List<TextBlock>();
        internal List<TextBlock> RightLabels = new List<TextBlock>();
        internal List<TextBlock> TopLabels = new List<TextBlock>();
        internal List<TextBlock> BottomLabels = new List<TextBlock>();

        private void CreateHorizontalRulers()
        {
            BottomCanvas.Children.Clear();
            TopCanvas.Children.Clear();
            CenterCanvas.Children.Clear();

            LeftLabels.Clear();
            RightLabels.Clear();
            TopLabels.Clear();
            BottomLabels.Clear();
               
            MajorIntervalX = BottomCanvas.ActualWidth / MajorThickCount;
            MinorIntervalX = BottomCanvas.ActualWidth / MinorThickCount;

            for (int i = 0; i <= MinorThickCount; i++)
            {
                var x = (MinorIntervalX * i);

                var minorThickB = CreateHorizontalThick(5);
                Canvas.SetLeft(minorThickB, x);
                Canvas.SetTop(minorThickB, 1); 
                BottomCanvas.Children.Add(minorThickB);
                 
                var minorThickT = CreateHorizontalThick(5);
                Canvas.SetLeft(minorThickT, x);
                Canvas.SetBottom(minorThickT, 1);
                TopCanvas.Children.Add(minorThickT);
            }

            for (int i = 0; i <= MajorThickCount; i++)
            {
                var x = (MajorIntervalX * i);

                var majorThickB = CreateHorizontalThick(20);
                Canvas.SetLeft(majorThickB, x);
                Canvas.SetTop(majorThickB, 1);
                BottomCanvas.Children.Add(majorThickB);

                var majorThickT = CreateHorizontalThick(20);
                Canvas.SetLeft(majorThickT, x);
                Canvas.SetBottom(majorThickT, 1);
                TopCanvas.Children.Add(majorThickT);
                 
                if (i > 0 && i < MajorThickCount)
                {
                    var gridline = CreateVerticalLine(x, CenterCanvas.ActualHeight); 
                    gridline.StrokeThickness = 1;
                    gridline.StrokeDashArray = new DoubleCollection() { 5 };
                    gridline.Stroke = new SolidColorBrush(Colors.Black); 
                    CenterCanvas.Children.Add(gridline);
                }
                  
                var labelB = new TextBlock();
                Canvas.SetLeft(labelB, x + 3);
                Canvas.SetTop(labelB, 8);
                BottomCanvas.Children.Add(labelB);
                BottomLabels.Add(labelB);

                var labelT = new TextBlock();
                Canvas.SetLeft(labelT, x + 3);
                Canvas.SetBottom(labelT, 8);
                TopCanvas.Children.Add(labelT);
                TopLabels.Add(labelT);
            } 
        }
        private void CreateVerticalRulers()
        {
            LeftCanvas.Children.Clear();
            RightCanvas.Children.Clear();

            MajorIntervalY = LeftCanvas.ActualHeight / MajorThickCount;
            MinorIntervalY = LeftCanvas.ActualHeight / MinorThickCount;

            for (int i = 0; i <= MinorThickCount; i++)
            {
                var y = (MinorIntervalY * i);

                var minorThickL = CreateVerticalThick(5); 
                Canvas.SetRight(minorThickL, 1);
                Canvas.SetTop(minorThickL, y);
                LeftCanvas.Children.Add(minorThickL);

                var minorThickR = CreateVerticalThick(5);
                Canvas.SetLeft(minorThickR, 1);
                Canvas.SetTop(minorThickR, y);
                RightCanvas.Children.Add(minorThickR);  
            }

            for (int i = 0; i <= MajorThickCount; i++)
            {
                var y = (MajorIntervalY * i);

                var majorThickL = CreateVerticalThick(20);
                Canvas.SetRight(majorThickL, 1);
                Canvas.SetTop(majorThickL, y);
                LeftCanvas.Children.Add(majorThickL);

                var majorThickR = CreateVerticalThick(20);
                Canvas.SetLeft(majorThickR, 1);
                Canvas.SetTop(majorThickR, y);
                RightCanvas.Children.Add(majorThickR);

                if (i < MajorThickCount && i > 0)
                {
                    var gridline = CreateHorizontalLine(y, CenterCanvas.ActualWidth); 
                    gridline.StrokeThickness = 1;
                    gridline.StrokeDashArray = new DoubleCollection() { 5 };
                    gridline.Stroke = new SolidColorBrush(Colors.Black); 
                    CenterCanvas.Children.Add(gridline); 
                }

                // skip last last labels on left/right rullers because they will overlap with top/bottom rulers
                if (i < MajorThickCount)
                {
                    var labelL = new TextBlock();
                    labelL.TextAlignment = TextAlignment.Left;
                    labelL.RenderTransform = new RotateTransform(90); 
                    Canvas.SetRight(labelL, -majorThickL.Width + 8);
                    Canvas.SetTop(labelL, y + 5);
                    LeftCanvas.Children.Add(labelL);
                    LeftLabels.Add(labelL);

                    var labelR = new TextBlock();
                    labelR.TextAlignment = TextAlignment.Right;
                    labelR.RenderTransform = new RotateTransform(90);
                    Canvas.SetLeft(labelR, majorThickR.Width + 8);
                    Canvas.SetTop(labelR, y + 5);
                    RightCanvas.Children.Add(labelR);
                    RightLabels.Add(labelR);
                } 
            } 
        }
        
        private void UpdateHorizontalRulers()
        {
            for (int i = 0; i < TopLabels.Count; i++)
            {
                var x = (MajorIntervalX * i);
                var pixel = new Point(MajorIntervalX * i, 0);
                var world = Diagram.ViewportToWorld(pixel);

                TopLabels[i].Text = world.X.ToString("0.");
                BottomLabels[i].Text = world.X.ToString("0.");

               // Debug.WriteLine("UpdateHorizontalRulers " +
               //" Point " + pixel.X +
               //" World " + world.X);
            }
        }
        private void UpdateVerticalRulers()
        {
            for (int i = 0; i < LeftLabels.Count; i++)
            {
                var x = (MajorIntervalY * i);
                var pixel = new Point(0, MajorIntervalY * i);
                var world = Diagram.ViewportToWorld(pixel);

                LeftLabels[i].Text = world.Y.ToString("0.");
                RightLabels[i].Text = world.Y.ToString("0.");

               // Debug.WriteLine("UpdateVerticalRulers " +
               //" Point " + pixel.Y +
               //" World " + world.Y);
            }
        }

        public XamDiagram Diagram
        {
            get { return (XamDiagram)GetValue(DiagramProperty); }
            set { SetValue(DiagramProperty, value); }
        }

        public static readonly DependencyProperty DiagramProperty =
            DependencyProperty.Register("Diagram", typeof(XamDiagram), typeof(DiagramRuler), new PropertyMetadata(null, (sender, e) =>
            {
                (sender as DiagramRuler).OnDiagramChanged(e.OldValue, e.NewValue);
            }));
         
        public void OnDiagramChanged(object oldValue, object newValue)
        {
            var oldDiagram = oldValue as XamDiagram;
            if (oldDiagram != null)
            {
                oldDiagram.PropertyChanged -= Diagram_PropertyChanged;
                oldDiagram.Loaded -= NewDiagram_Loaded;
            }

            var newDiagram = newValue as XamDiagram;
            if (newDiagram != null)
            {
                newDiagram.PropertyChanged += Diagram_PropertyChanged;
                newDiagram.Loaded += NewDiagram_Loaded;
            }
        }

        private void NewDiagram_Loaded(object sender, RoutedEventArgs e)
        {
            CreateRulers();
        }

        private void Diagram_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "WindowRect")
            {
                //Debug.WriteLine("Diagram_PropertyChanged " + e.PropertyName +
                //    " ViewportRect " + this.Diagram.ViewportRect +
                //    " ActualWidth " + this.Diagram.ActualWidth +
                //    " ActualHeight " + this.Diagram.ActualHeight+
                //    " WindowRect " + this.Diagram.WindowRect +
                //    //" WindowRectC " + rect +
                //    " WorldRect " + this.Diagram.WorldRect  );
                UpdateRulers();
            } 

        }

    }




}
